@@ -226,19 +226,30 @@ impl Ast {
         }
     }
 
-    pub fn substitute_symbols_from(&mut self, symbol_table: &HashMap<String, Ast>) {
+    pub fn substitute_symbols_from(&mut self,
+                                   symbol_table: &HashMap<String, Ast>,
+                                   lambda_vars: &mut HashSet<String>) {
         match &mut self.expr {
             Expr::Redex(left, right) => {
-                left.substitute_symbols_from(symbol_table);
-                right.substitute_symbols_from(symbol_table);
+                left.substitute_symbols_from(symbol_table, lambda_vars);
+                right.substitute_symbols_from(symbol_table, lambda_vars);
             },
-            Expr::LambdaTerm { var_name: _, body: lambda_body } => {
-                lambda_body.substitute_symbols_from(symbol_table);
+            Expr::LambdaTerm { var_name, body: lambda_body } => {
+                lambda_vars.insert(var_name.clone());
+                lambda_body.substitute_symbols_from(symbol_table, lambda_vars);
+                lambda_vars.remove(var_name);
             },
             Expr::Var {name, is_free} => if *is_free {
                 match symbol_table.get(name) {
                     None => return,
-                    Some(ast) => self.expr = ast.clone().expr,
+                    Some(ast) => {
+                        let mut new_ast = ast.clone();
+                        {
+                            let mut conversions = HashMap::new();
+                            new_ast.alpha_convert(lambda_vars, &mut conversions);
+                        }
+                        self.expr = new_ast.expr;
+                    }
                 };
             },
         }
